import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvAdaptPar(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvAdaptPar'
    bl_label = 'LB Adaptive Comfort Parameters'
    sv__ashrae_or_en15251_: StringProperty(name='_ashrae_or_en15251_', update=updateNode)
    sv__neutral_offset_: StringProperty(name='_neutral_offset_', update=updateNode)
    sv__avgm_or_runmean_: StringProperty(name='_avgm_or_runmean_', update=updateNode)
    sv__discr_or_cont_vel_: StringProperty(name='_discr_or_cont_vel_', update=updateNode)
    sv__cold_prevail_limit_: StringProperty(name='_cold_prevail_limit_', update=updateNode)
    sv__conditioning_: StringProperty(name='_conditioning_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_ashrae_or_en15251_').prop_name = 'sv__ashrae_or_en15251_'
        self.inputs.new('SvStringsSocket', '_neutral_offset_').prop_name = 'sv__neutral_offset_'
        self.inputs.new('SvStringsSocket', '_avgm_or_runmean_').prop_name = 'sv__avgm_or_runmean_'
        self.inputs.new('SvStringsSocket', '_discr_or_cont_vel_').prop_name = 'sv__discr_or_cont_vel_'
        self.inputs.new('SvStringsSocket', '_cold_prevail_limit_').prop_name = 'sv__cold_prevail_limit_'
        self.inputs.new('SvStringsSocket', '_conditioning_').prop_name = 'sv__conditioning_'
        self.sv_output_names = ['adapt_par']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['adapt_par']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_ashrae_or_en15251_', '_neutral_offset_', '_avgm_or_runmean_', '_discr_or_cont_vel_', '_cold_prevail_limit_', '_conditioning_']
        self.sv_input_types = ['bool', 'double', 'bool', 'bool', 'double', 'double']
        self.sv_input_defaults = [None, None, None, None, None, None]
        self.sv_input_access = ['item', 'item', 'item', 'item', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _ashrae_or_en15251_, _neutral_offset_, _avgm_or_runmean_, _discr_or_cont_vel_, _cold_prevail_limit_, _conditioning_):

        try:
            from ladybug_comfort.parameter.adaptive import AdaptiveParameter
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_comfort:\n\t{}'.format(e))
        
        
        adapt_par = AdaptiveParameter(_ashrae_or_en15251_, _neutral_offset_,
                                      _avgm_or_runmean_, _discr_or_cont_vel_,
                                      _cold_prevail_limit_, _conditioning_)
        

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvAdaptPar)

def unregister():
    bpy.utils.unregister_class(SvAdaptPar)
