import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvAnalysisPeriod(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvAnalysisPeriod'
    bl_label = 'LB Analysis Period'
    sv__start_month_: StringProperty(name='_start_month_', update=updateNode)
    sv__start_day_: StringProperty(name='_start_day_', update=updateNode)
    sv__start_hour_: StringProperty(name='_start_hour_', update=updateNode)
    sv__end_month_: StringProperty(name='_end_month_', update=updateNode)
    sv__end_day_: StringProperty(name='_end_day_', update=updateNode)
    sv__end_hour_: StringProperty(name='_end_hour_', update=updateNode)
    sv__timestep_: StringProperty(name='_timestep_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_start_month_').prop_name = 'sv__start_month_'
        self.inputs.new('SvStringsSocket', '_start_day_').prop_name = 'sv__start_day_'
        self.inputs.new('SvStringsSocket', '_start_hour_').prop_name = 'sv__start_hour_'
        self.inputs.new('SvStringsSocket', '_end_month_').prop_name = 'sv__end_month_'
        self.inputs.new('SvStringsSocket', '_end_day_').prop_name = 'sv__end_day_'
        self.inputs.new('SvStringsSocket', '_end_hour_').prop_name = 'sv__end_hour_'
        self.inputs.new('SvStringsSocket', '_timestep_').prop_name = 'sv__timestep_'
        self.sv_output_names = ['period', 'hoys', 'dates']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['period', 'hoys', 'dates']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_start_month_', '_start_day_', '_start_hour_', '_end_month_', '_end_day_', '_end_hour_', '_timestep_']
        self.sv_input_types = ['int', 'int', 'int', 'int', 'int', 'int', 'int']
        self.sv_input_defaults = [1, 1, 0, None, None, None, None]
        self.sv_input_access = ['item', 'item', 'item', 'item', 'item', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _start_month_, _start_day_, _start_hour_, _end_month_, _end_day_, _end_hour_, _timestep_):

        try:
            import ladybug.analysisperiod as ap
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))
        
        try:
            from ladybug_tools.sverchok import wrap_output
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        
        anp = ap.AnalysisPeriod(
            _start_month_, _start_day_, _start_hour_,
            _end_month_, _end_day_, _end_hour_, _timestep_)
        
        if anp:
            period = anp
            dates = wrap_output(anp.datetimes)
            hoys = anp.hoys

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvAnalysisPeriod)

def unregister():
    bpy.utils.unregister_class(SvAnalysisPeriod)
