import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvArithOp(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvArithOp'
    bl_label = 'LB Arithmetic Operation'
    sv__data_1: StringProperty(name='_data_1', update=updateNode)
    sv__data_2: StringProperty(name='_data_2', update=updateNode)
    sv__operator_: StringProperty(name='_operator_', update=updateNode)
    sv_type_: StringProperty(name='type_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_data_1').prop_name = 'sv__data_1'
        self.inputs.new('SvStringsSocket', '_data_2').prop_name = 'sv__data_2'
        self.inputs.new('SvStringsSocket', '_operator_').prop_name = 'sv__operator_'
        self.inputs.new('SvStringsSocket', 'type_').prop_name = 'sv_type_'
        self.sv_output_names = ['data']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['data']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_data_1', '_data_2', '_operator_', 'type_']
        self.sv_input_types = ['System.Object', 'System.Object', 'string', 'string']
        self.sv_input_defaults = [None, None, None, None]
        self.sv_input_access = ['list', 'list', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _data_1, _data_2, _operator_, type_):

        try:
            from ladybug_tools.sverchok import all_required_inputs
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        try:
            from itertools import izip as zip  # python 2
        except ImportError:
            pass  # some future time when Python upgrades to python 3
        
        
        if all_required_inputs(ghenv.Component):
            # build the arithmetic statement
            operator = '+' if _operator_ is None else _operator_
            statement = 'data_1 {} data_2'.format(operator)
        
            # perform the arithmetic operation
            data = []
            for data_1, data_2 in zip(_data_1, _data_2):
                result = eval(statement, {'data_1': data_1, 'data_2': data_2})  # I love Python!
        
                # try to replace the data collection type
                try:
                    result = result.duplicate()
                    if type_:
                        result.header.metadata['type'] = type_
                    elif 'type' in result.header.metadata:
                        del result.header.metadata['type']
                except AttributeError:
                    pass  # result was not a data collection; just return it anyway
                data.append(result)

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvArithOp)

def unregister():
    bpy.utils.unregister_class(SvArithOp)
