import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvHOY(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvHOY'
    bl_label = 'LB Calculate HOY'
    sv__month_: StringProperty(name='_month_', update=updateNode)
    sv__day_: StringProperty(name='_day_', update=updateNode)
    sv__hour_: StringProperty(name='_hour_', update=updateNode)
    sv__minute_: StringProperty(name='_minute_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_month_').prop_name = 'sv__month_'
        self.inputs.new('SvStringsSocket', '_day_').prop_name = 'sv__day_'
        self.inputs.new('SvStringsSocket', '_hour_').prop_name = 'sv__hour_'
        self.inputs.new('SvStringsSocket', '_minute_').prop_name = 'sv__minute_'
        self.sv_output_names = ['hoy', 'doy', 'date']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['hoy', 'doy', 'date']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_month_', '_day_', '_hour_', '_minute_']
        self.sv_input_types = ['int', 'int', 'int', 'int']
        self.sv_input_defaults = [None, None, None, None]
        self.sv_input_access = ['item', 'item', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _month_, _day_, _hour_, _minute_):

        try:
            from ladybug.dt import DateTime
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))
        
        month = _month_ or 9
        day = _day_ or 21
        hour = _hour_ if _hour_ is not None else 12
        minute = _minute_ or 0
        
        datetime = DateTime(month, day, hour, minute)
        hoy = datetime.hoy
        doy = datetime.doy
        date = datetime
        

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvHOY)

def unregister():
    bpy.utils.unregister_class(SvHOY)
