import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvConstrType(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvConstrType'
    bl_label = 'LB Construct Data Type'
    sv__name: StringProperty(name='_name', update=updateNode)
    sv__unit: StringProperty(name='_unit', update=updateNode)
    sv_categories_: StringProperty(name='categories_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_name').prop_name = 'sv__name'
        self.inputs.new('SvStringsSocket', '_unit').prop_name = 'sv__unit'
        self.inputs.new('SvStringsSocket', 'categories_').prop_name = 'sv_categories_'
        self.sv_output_names = ['type']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['type']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_name', '_unit', 'categories_']
        self.sv_input_types = ['string', 'string', 'string']
        self.sv_input_defaults = [None, None, None]
        self.sv_input_access = ['item', 'item', 'list']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _name, _unit, categories_):

        try:
            from ladybug.datatype.generic import GenericType
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))
        
        try:
            from ladybug_tools.sverchok import all_required_inputs
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        
        if all_required_inputs(ghenv.Component):
            # process the categories_ if they are supplied
            unit_descr = None
            if categories_ != []:
                unit_descr = {}
                for prop in categories_:
                    key, value = prop.split(':')
                    unit_descr[int(key)] = value.strip()
        
            type = GenericType(_name, _unit, unit_descr=unit_descr)

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvConstrType)

def unregister():
    bpy.utils.unregister_class(SvConstrType)
