import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvConstrLoc(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvConstrLoc'
    bl_label = 'LB Construct Location'
    sv__name_: StringProperty(name='_name_', update=updateNode)
    sv__latitude_: StringProperty(name='_latitude_', update=updateNode)
    sv__longitude_: StringProperty(name='_longitude_', update=updateNode)
    sv__time_zone_: StringProperty(name='_time_zone_', update=updateNode)
    sv__elevation_: StringProperty(name='_elevation_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_name_').prop_name = 'sv__name_'
        self.inputs.new('SvStringsSocket', '_latitude_').prop_name = 'sv__latitude_'
        self.inputs.new('SvStringsSocket', '_longitude_').prop_name = 'sv__longitude_'
        self.inputs.new('SvStringsSocket', '_time_zone_').prop_name = 'sv__time_zone_'
        self.inputs.new('SvStringsSocket', '_elevation_').prop_name = 'sv__elevation_'
        self.sv_output_names = ['location']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['location']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_name_', '_latitude_', '_longitude_', '_time_zone_', '_elevation_']
        self.sv_input_types = ['string', 'double', 'double', 'double', 'double']
        self.sv_input_defaults = [None, None, None, None, 0]
        self.sv_input_access = ['item', 'item', 'item', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _name_, _latitude_, _longitude_, _time_zone_, _elevation_):

        try:
            from ladybug.location import Location
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        location = Location(_name_, '-', '-', _latitude_, _longitude_, _time_zone_, _elevation_)
        

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvConstrLoc)

def unregister():
    bpy.utils.unregister_class(SvConstrLoc)
