import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvLegendParCategorized(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvLegendParCategorized'
    bl_label = 'LB Legend Parameters Categorized'
    sv__domain: StringProperty(name='_domain', update=updateNode)
    sv__colors: StringProperty(name='_colors', update=updateNode)
    sv_categories_: StringProperty(name='categories_', update=updateNode)
    sv_continuous_cols_: StringProperty(name='continuous_cols_', update=updateNode)
    sv_continuous_leg_: StringProperty(name='continuous_leg_', update=updateNode)
    sv_num_decimals_: StringProperty(name='num_decimals_', update=updateNode)
    sv_larger_smaller_: StringProperty(name='larger_smaller_', update=updateNode)
    sv_vert_or_horiz_: StringProperty(name='vert_or_horiz_', update=updateNode)
    sv_base_plane_: StringProperty(name='base_plane_', update=updateNode)
    sv_seg_height_: StringProperty(name='seg_height_', update=updateNode)
    sv_seg_width_: StringProperty(name='seg_width_', update=updateNode)
    sv_text_height_: StringProperty(name='text_height_', update=updateNode)
    sv_font_: StringProperty(name='font_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_domain').prop_name = 'sv__domain'
        self.inputs.new('SvStringsSocket', '_colors').prop_name = 'sv__colors'
        self.inputs.new('SvStringsSocket', 'categories_').prop_name = 'sv_categories_'
        self.inputs.new('SvStringsSocket', 'continuous_cols_').prop_name = 'sv_continuous_cols_'
        self.inputs.new('SvStringsSocket', 'continuous_leg_').prop_name = 'sv_continuous_leg_'
        self.inputs.new('SvStringsSocket', 'num_decimals_').prop_name = 'sv_num_decimals_'
        self.inputs.new('SvStringsSocket', 'larger_smaller_').prop_name = 'sv_larger_smaller_'
        self.inputs.new('SvStringsSocket', 'vert_or_horiz_').prop_name = 'sv_vert_or_horiz_'
        self.inputs.new('SvStringsSocket', 'base_plane_').prop_name = 'sv_base_plane_'
        self.inputs.new('SvStringsSocket', 'seg_height_').prop_name = 'sv_seg_height_'
        self.inputs.new('SvStringsSocket', 'seg_width_').prop_name = 'sv_seg_width_'
        self.inputs.new('SvStringsSocket', 'text_height_').prop_name = 'sv_text_height_'
        self.inputs.new('SvStringsSocket', 'font_').prop_name = 'sv_font_'
        self.sv_output_names = ['leg_par']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['leg_par']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_domain', '_colors', 'categories_', 'continuous_cols_', 'continuous_leg_', 'num_decimals_', 'larger_smaller_', 'vert_or_horiz_', 'base_plane_', 'seg_height_', 'seg_width_', 'text_height_', 'font_']
        self.sv_input_types = ['double', 'Color', 'string', 'bool', 'bool', 'int', 'bool', 'bool', 'Plane', 'double', 'double', 'double', 'string']
        self.sv_input_defaults = [None, None, None, None, None, None, None, None, None, None, None, None, None]
        self.sv_input_access = ['list', 'list', 'list', 'item', 'item', 'item', 'item', 'item', 'item', 'item', 'item', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _domain, _colors, categories_, continuous_cols_, continuous_leg_, num_decimals_, larger_smaller_, vert_or_horiz_, base_plane_, seg_height_, seg_width_, text_height_, font_):

        
        try:
            from ladybug.legend import LegendParametersCategorized
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))
        
        try:
            from ladybug_tools.togeometry import to_plane
            from ladybug_tools.sverchok import all_required_inputs
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        
        if all_required_inputs(ghenv.Component):
            if base_plane_:
                base_plane_ = to_plane(base_plane_)
            if len(categories_) == 0:
                categories_ = None
        
            leg_par = LegendParametersCategorized(
                domain=_domain, colors=_colors, category_names=categories_,
                base_plane=base_plane_)
        
            leg_par.continuous_colors = continuous_cols_
            leg_par.continuous_legend = continuous_leg_
            leg_par.decimal_count = num_decimals_
            leg_par.include_larger_smaller = larger_smaller_
            leg_par.vertical = vert_or_horiz_
            leg_par.segment_height = seg_height_
            leg_par.segment_width = seg_width_
            leg_par.text_height = text_height_
            leg_par.font = font_
        

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvLegendParCategorized)

def unregister():
    bpy.utils.unregister_class(SvLegendParCategorized)
