import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvMassArithOp(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvMassArithOp'
    bl_label = 'LB Mass Arithmetic Operation'
    sv__data: StringProperty(name='_data', update=updateNode)
    sv__operator_: StringProperty(name='_operator_', update=updateNode)
    sv_type_: StringProperty(name='type_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_data').prop_name = 'sv__data'
        self.inputs.new('SvStringsSocket', '_operator_').prop_name = 'sv__operator_'
        self.inputs.new('SvStringsSocket', 'type_').prop_name = 'sv_type_'
        self.sv_output_names = ['data']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['data']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_data', '_operator_', 'type_']
        self.sv_input_types = ['System.Object', 'string', 'System.Object']
        self.sv_input_defaults = [None, None, None]
        self.sv_input_access = ['list', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _data, _operator_, type_):

        try:
            from ladybug_tools.sverchok import all_required_inputs
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        
        if all_required_inputs(ghenv.Component):
            # build the arithmetic statement
            operator = '+' if _operator_ is None else _operator_
            statement = 'data {} data_i'.format(operator)
        
            # perform the arithmetic operation
            data = _data[0]
            for data_i in _data[1:]:
                data = eval(statement, {'data': data, 'data_i': data_i})  # I love Python!
        
            # try to replace the data collection type
            try:
                data = data.duplicate()
                if type_:
                    data.header.metadata['type'] = type_
                elif 'type' in data.header.metadata:
                    del data.header.metadata['type']
                if 'System' in data.header.metadata:
                    del data.header.metadata['System']
                if 'Zone' in data.header.metadata:
                    del data.header.metadata['Zone']
            except AttributeError:
                pass  # data was not a data collection; just return it anyway

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvMassArithOp)

def unregister():
    bpy.utils.unregister_class(SvMassArithOp)
