import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvSolarBodyPar(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvSolarBodyPar'
    bl_label = 'LB Solar Body Parameters'
    sv__posture_: StringProperty(name='_posture_', update=updateNode)
    sv__sharp_: StringProperty(name='_sharp_', update=updateNode)
    sv__body_az_: StringProperty(name='_body_az_', update=updateNode)
    sv__absorptivity_: StringProperty(name='_absorptivity_', update=updateNode)
    sv__emissivity_: StringProperty(name='_emissivity_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', '_posture_').prop_name = 'sv__posture_'
        self.inputs.new('SvStringsSocket', '_sharp_').prop_name = 'sv__sharp_'
        self.inputs.new('SvStringsSocket', '_body_az_').prop_name = 'sv__body_az_'
        self.inputs.new('SvStringsSocket', '_absorptivity_').prop_name = 'sv__absorptivity_'
        self.inputs.new('SvStringsSocket', '_emissivity_').prop_name = 'sv__emissivity_'
        self.sv_output_names = ['sol_body_par']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['sol_body_par']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['_posture_', '_sharp_', '_body_az_', '_absorptivity_', '_emissivity_']
        self.sv_input_types = ['string', 'double', 'double', 'double', 'double']
        self.sv_input_defaults = [None, None, None, None, None]
        self.sv_input_access = ['item', 'item', 'item', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, _posture_, _sharp_, _body_az_, _absorptivity_, _emissivity_):

        try:
            from ladybug_comfort.parameter.solarcal import SolarCalParameter
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_comfort:\n\t{}'.format(e))
        
        
        sol_body_par = SolarCalParameter(_posture_, _sharp_, _body_az_,
                                        _absorptivity_, _emissivity_)

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvSolarBodyPar)

def unregister():
    bpy.utils.unregister_class(SvSolarBodyPar)
