import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvSunpath(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvSunpath'
    bl_label = 'LB SunPath'
    sv_north_: StringProperty(name='north_', update=updateNode)
    sv__location: StringProperty(name='_location', update=updateNode)
    sv_hoys_: StringProperty(name='hoys_', update=updateNode)
    sv_dl_saving_: StringProperty(name='dl_saving_', update=updateNode)
    sv_solar_time_: StringProperty(name='solar_time_', update=updateNode)
    sv__center_pt_: StringProperty(name='_center_pt_', update=updateNode)
    sv__scale_: StringProperty(name='_scale_', update=updateNode)
    sv_projection_: StringProperty(name='projection_', update=updateNode)
    sv_daily_: StringProperty(name='daily_', update=updateNode)
    sv_data_: StringProperty(name='data_', update=updateNode)
    sv_statement_: StringProperty(name='statement_', update=updateNode)
    sv_legend_par_: StringProperty(name='legend_par_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', 'north_').prop_name = 'sv_north_'
        self.inputs.new('SvStringsSocket', '_location').prop_name = 'sv__location'
        self.inputs.new('SvStringsSocket', 'hoys_').prop_name = 'sv_hoys_'
        self.inputs.new('SvStringsSocket', 'dl_saving_').prop_name = 'sv_dl_saving_'
        self.inputs.new('SvStringsSocket', 'solar_time_').prop_name = 'sv_solar_time_'
        self.inputs.new('SvStringsSocket', '_center_pt_').prop_name = 'sv__center_pt_'
        self.inputs.new('SvStringsSocket', '_scale_').prop_name = 'sv__scale_'
        self.inputs.new('SvStringsSocket', 'projection_').prop_name = 'sv_projection_'
        self.inputs.new('SvStringsSocket', 'daily_').prop_name = 'sv_daily_'
        self.inputs.new('SvStringsSocket', 'data_').prop_name = 'sv_data_'
        self.inputs.new('SvStringsSocket', 'statement_').prop_name = 'sv_statement_'
        self.inputs.new('SvStringsSocket', 'legend_par_').prop_name = 'sv_legend_par_'
        self.sv_output_names = ['vectors', 'altitudes', 'azimuths', 'hoys', 'sun_pts', 'analemma', 'daily', 'compass', 'legend', 'title', 'color_pts']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['vectors', 'altitudes', 'azimuths', 'hoys', 'sun_pts', 'analemma', 'daily', 'compass', 'legend', 'title', 'color_pts']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['north_', '_location', 'hoys_', 'dl_saving_', 'solar_time_', '_center_pt_', '_scale_', 'projection_', 'daily_', 'data_', 'statement_', 'legend_par_']
        self.sv_input_types = ['System.Object', 'System.Object', 'double', 'System.Object', 'bool', 'Point3d', 'double', 'string', 'bool', 'System.Object', 'string', 'System.Object']
        self.sv_input_defaults = [None, None, None, None, None, None, 1, None, None, None, None, None]
        self.sv_input_access = ['item', 'item', 'list', 'item', 'item', 'item', 'item', 'item', 'item', 'list', 'item', 'list']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, north_, _location, hoys_, dl_saving_, solar_time_, _center_pt_, _scale_, projection_, daily_, data_, statement_, legend_par_):

        try:
            from ladybug_geometry.geometry2d.pointvector import Vector2D, Point2D
            from ladybug_geometry.geometry3d.pointvector import Point3D, Vector3D
            from ladybug_geometry.geometry3d.plane import Plane
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))
        
        try:
            from ladybug.sunpath import Sunpath
            from ladybug.compass import Compass
            from ladybug.graphic import GraphicContainer
            from ladybug.datacollection import HourlyContinuousCollection
            from ladybug.dt import Date
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))
        
        try:
            from ladybug_tools.config import conversion_to_meters
            from ladybug_tools.color import color_to_color
            from ladybug_tools.colorize import ColoredPoint
            from ladybug_tools.fromgeometry import from_polyline3d, from_polyline2d, \
                from_arc3d, from_vector3d, from_point3d, from_point2d
            from ladybug_tools.fromobjects import legend_objects, compass_objects
            from ladybug_tools.togeometry import to_vector2d, to_point2d, to_point3d
            from ladybug_tools.text import text_objects
            from ladybug_tools.sverchok import all_required_inputs, list_to_data_tree, \
                hide_output, show_output, schedule_solution
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        import math
        
        
        def draw_analemma_and_arcs(sp, datetimes, radius, center_pt3d):
            """Draw analemma and day arc Blender Ladybug geometry.
        
            Args:
                sp: Sunpath object for which geometry will be drawn.
                datetimes: A list of datetimes, which will be used to get days
                    if daily_ is True.
                radius: Number for the radius of the sun path.
                center_pt3d: Point3D for the center of the sun path.
        
            Returns:
                analemma: List of Blender Ladybug curves for the analemmas
                daily: List of Blender Ladybug curves for the daily arcs.
            """
            sp.daylight_saving_period = None  # set here so analemmas aren't messed up
        
            center_pt, z = Point2D(center_pt3d.x, center_pt3d.y), center_pt3d.z
            if not daily_:
                if projection_ is None:
                    analemma = [from_polyline3d(pline) for pline in sp.hourly_analemma_polyline3d(
                        center_pt3d, radius, True, solar_time_)]
                    daily = [from_arc3d(arc) for arc in sp.monthly_day_arc3d(center_pt3d, radius)]
                else:
                    analemma = [from_polyline2d(pline, z) for pline in sp.hourly_analemma_polyline2d(
                        projection_, center_pt, radius, True, solar_time_)]
                    daily = [from_polyline2d(arc, z) for arc in sp.monthly_day_polyline2d(
                        projection_, center_pt3d, radius)]
            else:
                analemma = []  # No Analemmas for a daily sun path
                doys = set(dt.doy for dt in datetimes)
                dates = [Date.from_doy(doy) for doy in doys]
                if projection_ is None:
                    daily = [from_arc3d(sp.day_arc3d(dat.month, dat.day, center_pt3d, radius))
                             for dat in dates]
                else:
                    daily = []
                    for dat in dates:
                        pline = sp.day_polyline2d(dat.month, dat.day, projection_, center_pt, radius)
                        daily.append(from_polyline2d(pline, z))
            return analemma, daily
        
        
        def draw_sun_positions(suns, radius, center_pt3d):
            """Draw Blender Ladybug points from a list of sun objects.
        
            Args:
                suns: A list of suns to be output as points
                radius: Number for the radius of the sun path.
                center_pt3d: Point3D for the center of the sun path.
        
            Returns:
                A list of Blender Ladybug points for sun positions
            """
                # get points for sun positions
            if projection_ is None:
                return [from_point3d(sun.position_3d(center_pt3d, radius)) for sun in suns]
            else:
                return [from_point2d(sun.position_2d(projection_, center_pt3d, radius), z)
                        for sun in suns]
        
        
        def title_text(data_col):
            """Get a text string for the title of the sunpath."""
            title_array = ['{} ({})'.format(data_col.header.data_type,
                                            data_col.header.unit)]
            for key, val in list(data_col.header.metadata.items()):
                title_array.append('{}: {}'.format(key, val))
            return '\n'.join(title_array)
        
        
        if all_required_inputs(ghenv.Component):
            # process all of the global inputs for the sunpath
            if north_ is not None:  # process the north_
                try:
                    north_ = math.degrees(
                        to_vector2d(north_).angle_clockwise(Vector2D(0, 1)))
                except AttributeError:  # north angle instead of vector
                    north_ = float(north_)
            else:
                north_ = 0
            if _center_pt_ is not None:  # process the center point into a Point2D
                center_pt, center_pt3d = to_point2d(_center_pt_), to_point3d(_center_pt_)
                z = center_pt3d.z
            else:
                center_pt, center_pt3d = Point2D(), Point3D()
                z = 0
            _scale_ = 1 if _scale_ is None else _scale_ # process the scale into a radius
            radius = (100 * _scale_) / conversion_to_meters()
            solar_time_ = False if solar_time_ is None else solar_time_  # process solat time
        
            # create a intersection of the input hoys_ and the data hoys
            if len(data_) > 0 and len(hoys_) > 0:
                all_aligned = all(data_[0].is_collection_aligned(d) for d in data_[1:])
                assert all_aligned, 'All collections input to data_ must be aligned for ' \
                    'each Sunpath.\nGrafting the data_ and suplying multiple grafted ' \
                    '_center_pt_ can be used to view each data on its own path.'
                if statement_ is not None:
                    data_ = HourlyContinuousCollection.filter_collections_by_statement(
                        data_, statement_)
                data_hoys = set(dt.hoy for dt in data_[0].datetimes)
                hoys_ = list(data_hoys.intersection(set(hoys_)))
        
            # initialize sunpath based on location
            sp = Sunpath.from_location(_location, north_, dl_saving_)
        
            # process all of the input hoys into altitudes, azimuths and vectors
            altitudes, azimuths, datetimes, moys, hoys, vectors, suns = [], [], [], [], [], [], []
            for hoy in hoys_:
                sun = sp.calculate_sun_from_hoy(hoy, solar_time_)
                if sun.is_during_day:
                    altitudes.append(sun.altitude)
                    azimuths.append(sun.azimuth)
                    datetimes.append(sun.datetime)
                    moys.append(sun.datetime.moy)
                    hoys.append(sun.datetime.hoy)
                    vectors.append(from_vector3d(sun.sun_vector))
                    suns.append(sun)
        
            if len(data_) > 0 and len(hoys_) > 0:  # build a sunpath for each data collection
                title, all_sun_pts, all_analemma, all_daily, all_compass, all_col_pts, all_legends = \
                    [], [], [], [], [], [], []
                for i, data in enumerate(data_):
                    try:  # sense when several legend parameters are connected
                        lpar = legend_par_[i]
                    except IndexError:
                        lpar = None if len(legend_par_) == 0 else legend_par_[-1]
        
                    # move the center point so sun paths are not on top of one another
                    fac = i* radius * 3
                    center_pt_i = Point2D(center_pt.x + fac, center_pt.y)
                    center_pt3d_i = Point3D(center_pt3d.x + fac, center_pt3d.y, center_pt3d.z)
        
                    # create the ladybug compass object
                    lb_compass = Compass(radius, center_pt_i, north_)
        
                    # create a graphic container to generate colors and legends
                    n_data = data.filter_by_moys(moys)  # filter data collection by sun-up hours
                    graphic = GraphicContainer(
                        n_data.values, lb_compass.min_point3d(z), lb_compass.max_point3d(z),
                        lpar, n_data.header.data_type, n_data.header.unit)
                    all_legends.append(legend_objects(graphic.legend))
                    title.append(text_objects(
                        title_text(n_data), graphic.lower_title_location,
                        graphic.legend_parameters.text_height, graphic.legend_parameters.font))
        
                    # create points, analemmas, daily arcs, and compass geometry
                    sun_pts_init = draw_sun_positions(suns, radius, center_pt3d_i)
                    analemma_i, daily_i = draw_analemma_and_arcs(sp, datetimes, radius, center_pt3d_i)
                    compass_i = compass_objects(lb_compass, z, None, projection_,
                                                graphic.legend_parameters.font)
                    all_analemma.append(analemma_i)
                    all_daily.append(daily_i)
                    all_compass.append(compass_i)
        
                    # produce a visualization of colored points
                    cols = [color_to_color(col) for col in graphic.value_colors]
                    col_pts = []
                    for pt, col in zip(sun_pts_init, cols):
                        col_pt = ColoredPoint(pt)
                        col_pt.color = col
                        col_pts.append(col_pt)
                    all_sun_pts.append(sun_pts_init)
                    all_col_pts.append(col_pts)
        
                # convert all nested lists to data trees
                sun_pts = list_to_data_tree(all_sun_pts)
                analemma = list_to_data_tree(all_analemma)
                daily = list_to_data_tree(all_daily)
                compass = list_to_data_tree(all_compass)
                legend = list_to_data_tree(all_legends)
        
                # do some acrobatics to get the colored points to display
                # CWM: I don't know why we have to re-schedule the solution but this is the
                # only way I found to get the colored points to appear (redraw did not work).
                color_pts = list_to_data_tree(all_col_pts)
                hide_output(ghenv.Component, 5)
                schedule_solution(ghenv.Component, 2)
            else:  # no data connected; just output one sunpath
                sun_pts = draw_sun_positions(suns, radius, center_pt3d)
                analemma, daily = draw_analemma_and_arcs(sp, datetimes, radius, center_pt3d)
                font = legend_par_[0].font if len(legend_par_) != 0 and \
                    legend_par_[0] is not None else 'Arial'
                compass = compass_objects(Compass(radius, center_pt, north_), z, None, projection_, font)
                if _location.city:
                    title = text_objects(
                        'city: {}'.format(_location.city),
                        Plane(o=center_pt3d.move(Vector3D(-radius * 1.25, -radius * 1.25))),
                        radius / 15, font)
                show_output(ghenv.Component, 5)
        

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvSunpath)

def unregister():
    bpy.utils.unregister_class(SvSunpath)
