import bpy
import ladybug_tools.helper
from bpy.props import StringProperty
from sverchok.node_tree import SverchCustomTreeNode
from sverchok.data_structure import updateNode, zip_long_repeat

ghenv = ladybug_tools.helper.ghenv

class SvWindRose(bpy.types.Node, SverchCustomTreeNode):
    bl_idname = 'SvWindRose'
    bl_label = 'LB Wind Rose'
    sv_north_: StringProperty(name='north_', update=updateNode)
    sv__data: StringProperty(name='_data', update=updateNode)
    sv__wind_direction: StringProperty(name='_wind_direction', update=updateNode)
    sv__dir_count_: StringProperty(name='_dir_count_', update=updateNode)
    sv__center_pt_: StringProperty(name='_center_pt_', update=updateNode)
    sv__show_calmrose_: StringProperty(name='_show_calmrose_', update=updateNode)
    sv__show_freq_: StringProperty(name='_show_freq_', update=updateNode)
    sv__freq_dist_: StringProperty(name='_freq_dist_', update=updateNode)
    sv__freq_hours_: StringProperty(name='_freq_hours_', update=updateNode)
    sv__max_freq_lines_: StringProperty(name='_max_freq_lines_', update=updateNode)
    sv_legend_par_: StringProperty(name='legend_par_', update=updateNode)
    sv_statement_: StringProperty(name='statement_', update=updateNode)
    sv_period_: StringProperty(name='period_', update=updateNode)

    def sv_init(self, context):
        self.inputs.new('SvStringsSocket', 'north_').prop_name = 'sv_north_'
        self.inputs.new('SvStringsSocket', '_data').prop_name = 'sv__data'
        self.inputs.new('SvStringsSocket', '_wind_direction').prop_name = 'sv__wind_direction'
        self.inputs.new('SvStringsSocket', '_dir_count_').prop_name = 'sv__dir_count_'
        self.inputs.new('SvStringsSocket', '_center_pt_').prop_name = 'sv__center_pt_'
        self.inputs.new('SvStringsSocket', '_show_calmrose_').prop_name = 'sv__show_calmrose_'
        self.inputs.new('SvStringsSocket', '_show_freq_').prop_name = 'sv__show_freq_'
        self.inputs.new('SvStringsSocket', '_freq_dist_').prop_name = 'sv__freq_dist_'
        self.inputs.new('SvStringsSocket', '_freq_hours_').prop_name = 'sv__freq_hours_'
        self.inputs.new('SvStringsSocket', '_max_freq_lines_').prop_name = 'sv__max_freq_lines_'
        self.inputs.new('SvStringsSocket', 'legend_par_').prop_name = 'sv_legend_par_'
        self.inputs.new('SvStringsSocket', 'statement_').prop_name = 'sv_statement_'
        self.inputs.new('SvStringsSocket', 'period_').prop_name = 'sv_period_'
        self.sv_output_names = ['mesh', 'compass', 'orient_line', 'freq_line', 'legend', 'title', 'prevailing', 'frequency', 'avg_val', 'data']
        for name in self.sv_output_names:
            self.outputs.new('SvStringsSocket', name)

    def process(self):
        if not any(socket.is_linked for socket in self.outputs):
            return

        self.sv_output_names = ['mesh', 'compass', 'orient_line', 'freq_line', 'legend', 'title', 'prevailing', 'frequency', 'avg_val', 'data']
        for name in self.sv_output_names:
            setattr(self, '{}_out'.format(name), [])
        self.sv_input_names = ['north_', '_data', '_wind_direction', '_dir_count_', '_center_pt_', '_show_calmrose_', '_show_freq_', '_freq_dist_', '_freq_hours_', '_max_freq_lines_', 'legend_par_', 'statement_', 'period_']
        self.sv_input_types = ['System.Object', 'System.Object', 'System.Object', 'int', 'Point3d', 'bool', 'bool', 'int', 'int', 'int', 'System.Object', 'string', 'System.Object']
        self.sv_input_defaults = [None, None, None, None, None, None, None, None, None, None, None, None, None]
        self.sv_input_access = ['item', 'list', 'item', 'item', 'item', 'item', 'item', 'item', 'item', 'item', 'list', 'item', 'item']
        sv_inputs_nested = []
        for name in self.sv_input_names:
            sv_inputs_nested.append(self.inputs[name].sv_get())
        for sv_input_nested in zip_long_repeat(*sv_inputs_nested):
            for sv_input in zip_long_repeat(*sv_input_nested):
                sv_input = list(sv_input)
                for i, value in enumerate(sv_input):
                    if self.sv_input_access[i] == 'list':
                        if isinstance(value, (list, tuple)):
                            values = value
                        else:
                            values = [value]
                        value = [self.sv_cast(v, self.sv_input_types[i], self.sv_input_defaults[i]) for v in values]
                        if value == [None]:
                            value = []
                        sv_input[i] = value
                    else:
                        sv_input[i] = self.sv_cast(value, self.sv_input_types[i], self.sv_input_defaults[i])
                self.process_ladybug(*sv_input)
        for name in self.sv_output_names:
            value = getattr(self, '{}_out'.format(name))
            # Not sure if this hack is correct, will find out when more nodes are generated
            #if len(value) == 0 or not isinstance(value[0], (list, tuple)):
            #    value = [value]
            self.outputs[name].sv_set(value)

    def sv_cast(self, value, data_type, default):
        result = default if isinstance(value, str) and value == '' else value
        if result is None and data_type == 'bool':
            return False
        return result

    def process_ladybug(self, north_, _data, _wind_direction, _dir_count_, _center_pt_, _show_calmrose_, _show_freq_, _freq_dist_, _freq_hours_, _max_freq_lines_, legend_par_, statement_, period_):

        import math
        
        try:
            from ladybug.datacollection import HourlyContinuousCollection
            from ladybug.windrose import WindRose
            from ladybug.datatype.speed import Speed
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug:\n\t{}'.format(e))
        
        try:
            from ladybug_geometry.geometry2d.pointvector import Point2D, Vector2D
            from ladybug_geometry.geometry3d.pointvector import Point3D
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))
        
        try:
            from ladybug_tools.config import conversion_to_meters
            from ladybug_tools.togeometry import to_point3d, to_vector2d
            from ladybug_tools.fromgeometry import from_mesh2d, from_linesegment2d, \
                from_polygon2d
            from ladybug_tools.text import text_objects
            from ladybug_tools.fromobjects import legend_objects, compass_objects
            from ladybug_tools.sverchok import \
                all_required_inputs, list_to_data_tree, data_tree_to_list
        except ImportError as e:
            raise ImportError('\nFailed to import ladybug_tools:\n\t{}'.format(e))
        
        
        def title_text(data_col):
            """Get a text string for the title of the windrose."""
            title_array = ['{} ({})'.format(data_col.header.data_type,
                                            data_col.header.unit)]
            for key, val in list(data_col.header.metadata.items()):
                title_array.append('{}: {}'.format(key, val))
            title_array.append('period: {}'.format(data_col.header.analysis_period))
            return '\n'.join(title_array)
        
        
        if all_required_inputs(ghenv.Component):
            # Apply any analysis periods and conditional statement to the input collections
            if period_ is not None:
                _data = [dat.filter_by_analysis_period(period_) for dat in _data]
                _wind_direction = _wind_direction.filter_by_analysis_period(period_)
        
            if statement_ is not None and statement_.strip() != "":
                _fdata = HourlyContinuousCollection.filter_collections_by_statement(
                    _data + [_wind_direction], statement_)
                _data = _fdata[:-1]
                _wind_direction = _fdata[-1]
        
            # filter zero speed values out of collections if speed is input
            pattern = []
            filt_wind_dir = _wind_direction
            for dat in _data:
                if isinstance(dat.header.data_type, Speed):
                    for val in dat.values:
                        pat = True if val > 1e-10 else False
                        pattern.append(pat)
                    break
            if len(pattern) != 0:
                for i, dat in enumerate(_data):
                    if not isinstance(dat.header.data_type, Speed):
                        _data[i] = dat.filter_by_pattern(pattern)
                filt_wind_dir = _wind_direction.filter_by_pattern(pattern)
        
            # set defaults and check errors in dir_count and north input
            if _dir_count_ is None:
                _dir_count_ = 16
            assert _dir_count_ > 2, 'The number of directions must be greater then three to ' \
                'plot the wind rose. Got: {}'.format(_dir_count_)
        
            if north_ is not None:  # process the north_
                try:
                    north_ = math.degrees(to_vector2d(north_).angle_clockwise(Vector2D(0, 1)))
                except AttributeError:  # north angle instead of vector
                    north_ = float(north_)
                    assert -360.0 <= north_ <= 360.0, 'The north orientation must be greater ' \
                        'then -360 and less then 360 to plot the wind rose. ' \
                        'Got: {}'.format(north_)
            else:
                north_ = 0.0
        
            # set default values for the chart dimensions
            _center_pt_ = to_point3d(_center_pt_) if _center_pt_ is not None else Point3D()
            center_pt_2d = Point2D(_center_pt_.x, _center_pt_.y)
        
            # set defaults frequency hours and distance so chart is same scale as other LB plots
            if _freq_hours_ is None:
                _freq_hours_ = 50.0
            if _freq_dist_ is None:
                _freq_dist_ = 5.0 / conversion_to_meters()
        
            # set default show_freq and _show_calmrose_
            _show_calmrose_ = False if _show_calmrose_ is None else _show_calmrose_
            _show_freq_ = True if _show_freq_ is None else _show_freq_
        
            # set up empty lists of objects to be filled
            all_wind_avg_val = []
            all_wind_frequency = []
            all_mesh = []
            all_compass = []
            all_orient_line = []
            all_freq_line = []
            all_legends = []
            all_title = []
        
            # Calculate _max_freq_lines_ if it's not already set, to use to
            # determine spacing for multiple plots.
            if len(_data) > 1 and _max_freq_lines_ is None:
                max_freqs = []
                for i, _data_item in enumerate(_data):
                    win_dir = _wind_direction if isinstance(_data_item.header.data_type, Speed) \
                        else filt_wind_dir
                    w = WindRose(win_dir, _data_item, _dir_count_)
                    if _freq_hours_ is not None:
                        w.frequency_hours = _freq_hours_
                    if _freq_dist_ is not None:
                        w.frequency_spacing_distance = _freq_dist_
                    max_freqs.append(w.frequency_intervals_compass)
                _max_freq_lines_ = max(max_freqs)
        
            # Plot the windroses
            for i, speed_data in enumerate(_data):
        
                # Make the windrose
                win_dir = _wind_direction if isinstance(speed_data.header.data_type, Speed) \
                    else filt_wind_dir
                windrose = WindRose(win_dir, speed_data, _dir_count_)
        
                if len(legend_par_) > 0:
                    try:  # sense when several legend parameters are connected
                        lpar = legend_par_[i]
                    except IndexError:
                        lpar = legend_par_[-1]
                    windrose.legend_parameters = lpar
        
                if _freq_hours_ is not None:
                    windrose.frequency_hours = _freq_hours_
        
                # Add and check optional visualization parameters
                if _max_freq_lines_ is not None:
                    windrose.frequency_intervals_compass = _max_freq_lines_
        
                if _freq_dist_ is not None:
                    windrose.frequency_spacing_distance = _freq_dist_
        
                windrose.north = north_
                windrose.show_freq = _show_freq_
                if isinstance(speed_data.header.data_type, Speed):
                    windrose.show_zeros = _show_calmrose_
                windrose.base_point = Point2D(center_pt_2d.x, center_pt_2d.y)
        
                # Make the mesh
                mesh = from_mesh2d(windrose.colored_mesh, _center_pt_.z)
        
                # Make the graphic outputs
                legend = legend_objects(windrose.legend)
                title = text_objects(title_text(speed_data),
                                     windrose.container.lower_title_location,
                                     windrose.legend_parameters.text_height,
                                     windrose.legend_parameters.font)
                compass = compass_objects(windrose.compass, _center_pt_.z, None)
                orient_line = [from_linesegment2d(seg, _center_pt_.z)
                                    for seg in windrose.orientation_lines]
                if _show_freq_:
                    freq_line = [from_polygon2d(poly) for poly in windrose.frequency_lines]
        
                # move the center point so wind roses are not on top of one another
                fac = (i + 1) * windrose.compass_radius * 3
                center_pt_2d = Point2D(_center_pt_.x + fac, _center_pt_.y)
        
                all_mesh.append(mesh)
                all_compass.append(compass)
                all_orient_line.append(orient_line)
                all_freq_line.append(freq_line)
                all_legends.append(legend)
                all_title.append(title)
                
                # compute the average values
                wind_avg_val = []
                for bin in windrose.histogram_data:
                    try:
                        wind_avg_val.append(sum(bin) / len(bin))
                    except ZeroDivisionError:
                        wind_avg_val.append(0)
                all_wind_avg_val.append(wind_avg_val)
                all_wind_frequency.append([len(bin) for bin in windrose.histogram_data])
        
            # convert nested lists into data trees
            mesh = list_to_data_tree(all_mesh)
            compass = list_to_data_tree(all_compass)
            orient_line = list_to_data_tree(all_orient_line)
            freq_line = list_to_data_tree(all_freq_line)
            legend = list_to_data_tree(all_legends)
            title = list_to_data_tree(all_title)
            avg_val = list_to_data_tree(all_wind_avg_val)
            frequency = list_to_data_tree(all_wind_frequency)
        
            # output prevailing direction and processed data
            prevailing = windrose.prevailing_direction
            data = _data

        for name in self.sv_output_names:
            if name in locals():
                getattr(self, '{}_out'.format(name)).append([locals()[name]])


def register():
    bpy.utils.register_class(SvWindRose)

def unregister():
    bpy.utils.unregister_class(SvWindRose)
